import time

import structlog
from aiohttp.web import Request, middleware
from structlog.contextvars import clear_contextvars

slogger = structlog.get_logger(__name__)


@middleware
async def request_logging_middleware(request: Request, handler):
    clear_contextvars()
    request_rule = request.match_info.get_info().get("formatter")
    if request_rule is not None:
        request_rule = request_rule.replace("{", "<").replace("}", ">")
    start = time.monotonic()
    response = await handler(request)
    duration = time.monotonic() - start
    slogger.debug(
        "request.finished",
        path=request.path,
        rule=request_rule,
        method=request.method,
        status=response.status,
        duration=round(duration * 1000, 3),
    )
    return response
