import time

import structlog
from flask import request
from structlog.contextvars import clear_contextvars

slogger = structlog.get_logger(__name__)


def before_request_log_setup():
    # Old school way of clearing the logger context. Does not clear anything
    # stored in contextvars.
    slogger.new()

    # New school way. Also clear the contextvars.
    clear_contextvars()
    request.start = time.monotonic()


def after_request_log_teardown(response):
    duration = time.monotonic() - request.start
    slogger.debug(
        "request.finished",
        path=request.path,
        rule=request.url_rule.rule if request.url_rule else None,  # None for 404
        method=request.method,
        status=response.status_code,
        duration=round(duration * 1000, 3),
    )
    clear_contextvars()
    return response
