import os
from argparse import OPTIONAL, SUPPRESS, ZERO_OR_MORE, Action, RawTextHelpFormatter


class EnvDefault(Action):
    """Argparse action for receiving environment variables as defaults for
    arguments.

    Idea taken from https://stackoverflow.com/a/10551190/1143231
    """

    def __init__(self, env, type=str, required=True, default=None, **kwargs):
        if env in os.environ:
            default = type(os.environ[env])
        if default is not None:
            required = False
        super().__init__(default=default, required=required, type=type, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, values)


class DefaultsHelpFormatter(RawTextHelpFormatter):
    """Help message formatter which retains formatting of all help text.

    It also appends the default value of an argument to its help text
    in case a help text is provided and the default value is not None.
    """

    def _get_help_string(self, action):
        help = action.help
        if action.default is not SUPPRESS and action.default is not None:
            defaulting_nargs = [OPTIONAL, ZERO_OR_MORE]
            if action.option_strings or action.nargs in defaulting_nargs:
                help += "\nDefault: %(default)s"
        return help
